using IWS5.Core.DTO.IAdmin.Servers;
using IWS5.Core.Models;

namespace IWS5.Core.Services.IAdmin;

public interface IServersService
{
    Task<CodeResponse<List<ServerDetailsDTO>>> GetServersAsync();
    Task<CodeResponse<ServerDetailsDTO>> GetServerDetailsAsync(string hostName);
    Task<CodeResponse<ServerDetailsDTO>> CreateServerAsync(CreateServerRequestDTO createServerRequest);
    Task<CodeResponse<ServerDetailsDTO>> UpdateServerAsync(string hostName, UpdateServerRequestDTO updateServerRequest);
    Task<bool> DeleteServerAsync(string hostName);
}
